<?php
/* Copyright 2007, 2008 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class PhpThumbComponent extends Object {
	private static $CACHE;
	private static $CACHE_SUBDIR = 'thumbs';
	private static $MAX_AGE = 1296000; // 15 days

	public function startup(&$controller) {
		// I hate this way of not letting expressions as default values
		self::$CACHE = IMAGES.self::$CACHE_SUBDIR.DS;
		
		// Check cache directory
		if (! file_exists(self::$CACHE)) {
			if (! mkdir(self::$CACHE)) {
				trigger_error('Cannot create '.self::$CACHE, E_USER_ERROR);
				return;
			}
		} else if (! is_dir(self::$CACHE)) {
			trigger_error(self::$CACHE.' is not a directory', E_USER_ERROR);
			return;
		}
		if (! is_writable(self::$CACHE)) {
			trigger_error(self::$CACHE.' is not writable', E_USER_ERROR);
			return;
		}
		
		// Prune
		$dh = opendir(self::$CACHE);
		while (($file = readdir($dh)) !== false) {
            if (is_file(self::$CACHE.$file)
				&& filemtime(self::$CACHE.$file)+self::$MAX_AGE < time())
				if (! unlink(self::$CACHE.$file))
					$this->log('Failed to delete obsolete thumb file '.$file);
        }
		closedir($dh);
	}

	/** Returns the url to the thumbnail of a given image.
	 * @param width The maximum thumbnail width. Any value below 0 will be
	 * replaced by the configured maximum size.
	 * @param height The maximum thumbnail height. Any value below 0 will be
	 * replaced by the configured maximum size.
	 * @return The thumbnail path, relative to the IMAGES_URL directory
	 */
	public function getThumbnail($image, $width = 0, $height = 0) {
		if (! is_file($image)) {
			trigger_error($image.' is not a file', E_USER_WARNING);
			return false;
		}
		
		if ($width <= 0)
			$width = Configure::read('Demyo.maxThumbWidth');
		if ($height <= 0)
			$height = Configure::read('Demyo.maxThumbHeight');

		App::import('Vendor', 'phpThumb', array('file' => 'phpThumb'.DS.'phpthumb.class.php'));

		$outputFilename = md5($image.$width.$height).'.jpg';
		if (file_exists(self::$CACHE.$outputFilename))
			return 'thumbs/'.$outputFilename;
		
		$phpThumb = new phpThumb();
        $phpThumb->setSourceFilename($image);
		$phpThumb->setParameter('config_allow_src_above_docroot', true);
        $phpThumb->setParameter('w',$width);
        $phpThumb->setParameter('h',$height);
		$phpThumb->setParameter('q',75);
		$phpThumb->setParameter('config_output_format', 'jpeg');
		if($phpThumb->generateThumbnail()){
			if(!$phpThumb->RenderToFile(self::$CACHE.$outputFilename)){
				trigger_error('Failed to render thumbnail for '.basename($image), E_USER_WARNING);
				return false;
			}
			
			if (strtoupper(substr(PHP_OS, 0, 3)) !== 'WIN')
				chmod(self::$CACHE.$outputFilename, 0666); // See bug #91
		} else {
			trigger_error('Failed to generate thumbnail for '.basename($image). ' : '.$phpThumb->fatalerror.(implode("\n*", $phpThumb->debugmessages)), E_USER_WARNING);
			return false;
		}

		return 'thumbs/'.$outputFilename;
	}

}

?>