<?php
/* Copyright 2007, 2008 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class LibraryController extends AppController {
	public $uses = array('Publisher','Collection','Binding','Author','Artist'
		,'Writer','Colorist','Tag','Album','Series','Image', 'Derivative'
		,'DerivativeType', 'Source', 'Borrower', 'AlbumsBorrower');
	public $helpers = array('Chart', 'Form', 'Form', 'Html', 'JQuery', 'List', 'Text', 'xml');
	public $components = array('ImportExport');
	private $error_log = null;

	public function index() {
		$this->redirect('/');
	}

	public function stats() {
		$MAX_CHAR_RESULTS = 15;
		
		// Counts
		$counts['Publisher'] = $this->Publisher->findCount();
		$counts['Collection'] = $this->Collection->findCount();
		$counts['Author'] = $this->Author->findCount();
		$counts['Series'] = $this->Series->findCount();
		$counts['Album'] = $this->Album->findCount();
		$counts['Album_orig'] = $this->Album->findCount('Album.this_edition = Album.first_edition AND Album.first_edition != \'0000-00-00\' AND Album.first_edition IS NOT NULL');
		$counts['Album_lended'] = $this->AlbumsBorrower->findLendedCount();
		$counts['Image'] = $this->Image->findCount();
		$counts['Tag'] = $this->Tag->findCount();
		$counts['Derivative'] = $this->Derivative->findCount();
		$this->set('counts', $counts);
		
		// Prices
		$worth['Album'] = $this->Album->getWorth();
		$worth['Derivative'] = $this->Derivative->getWorth();
		$this->set('worth', $worth);
		
		$albumTable = $this->Album->tablePrefix . $this->Album->table;
		$charts = array();
		
		// Edited albums, per year
		list($data, $labels, $charts['thisEd']['notShown']) = $this->chartYears(
			$this->Album->query("SELECT YEAR(this_edition) AS year ,COUNT(*) AS count FROM $albumTable GROUP BY year ORDER BY year DESC"));
		$charts['thisEd']['data'] = array(array_reverse(stringArrayToIntArray($data)));
		$charts['thisEd']['labels'] =  array('x' => (array_reverse($labels)));
		
		// Purchased albums per year
		list($data, $labels, $charts['purchase']['notShown']) = $this->chartYears(
			$this->Album->query("SELECT YEAR(purchase_date) AS year ,COUNT(*) AS count FROM $albumTable GROUP BY year ORDER BY year DESC"));
		$charts['purchase']['data'] = array(array_reverse(stringArrayToIntArray($data)));
		$charts['purchase']['labels'] =  array('x' => (array_reverse($labels)));
		
		$this->set('charts', $charts);
		
		// Load logs
		$logs['debug'] = file_get_contents(LOGS.'debug.log');
		$logs['error'] = file_get_contents(LOGS.'error.log');
		$this->set('logs', $logs);
	}
	
	public function search() {}
	
	public function dosearch() {
		$this->layout = 'ajax';
		
		if (! isset($this->data))
			return;
		if (get_magic_quotes_gpc())
			$input = $this->data['Search']['query'];
		else
			$input = addslashes($this->data['Search']['query']);
		
		$input = str_replace('*', '%', $input);
		$input = str_replace('?', '_', $input);
		if ($input[0] != '%')
			$input = '%'.$input;
		if ($input[strlen($input)-1] != '%')
			$input .= '%';
		
		$results = array();
		
		// Not using an index might look ill-optimised but the fields that are
		// searched are so small it wouldn't give any benefit besides looking
		// more enterprisy
		$modelNames = array('Series', 'Album', 'Author', 'Publisher', 'Collection');
		$conditions = array("name LIKE '$input'", "title LIKE '$input'",
			"name LIKE '$input' OR fname LIKE '$input' OR nickname LIKE '$input'", "name LIKE '$input'",
			"name LIKE '$input'");
		$fields = array(array('id', 'name'), array('id', 'title'),
			array('id', 'name', 'fname', 'nickname'), array('id', 'name'), array('id', 'name'));
		$orders = array('name ASC', 'title ASC', 'name ASC', 'name ASC', 'name ASC');
		$len = sizeof($modelNames);
		for ($i = 0; $i < $len; $i++)
			if (isset($this->data['Search'][$modelNames[$i]]) && $this->data['Search'][$modelNames[$i]] === 'on')
			$results[$modelNames[$i]] = Set::extract(
				$this->{$modelNames[$i]}->findAll($conditions[$i], $fields[$i], $orders[$i], null, 1, -1),
				'{n}.'.$modelNames[$i]);
		
		$this->set('results', $results);
	}
	
	private function chartYears(&$resultSet, $maxResults = 15) {
		$notshown = 0;
		$shown = 0;
		$counts = array();
		$labels = array();
		
		// Flatten and restrict
		$i = 0;
		foreach($resultSet as $result) {
			if ($result['0']['year'] == 0) {
				$notshown += $result['0']['count'];
				continue;
			}
			$i++;
			if ($i <= $maxResults) {
				$counts[] = $result['0']['count'];
				$labels[] = $result['0']['year'];
				$shown += $result['0']['count'];
			} else
				$notshown += $result['0']['count'];
		}
		
		return array($counts, $labels, $notshown);
	}
	
	public function export() {
		if (! isset($this->params['form']['method']))
			return;
		
		$this->layout = 'raw';
		
		$withResources = isset($this->params['form']['withResources']) && $this->params['form']['withResources'] == 'on';
		
		if (($exported = $this->ImportExport->export($this->params['form']['method'], $withResources)) === false) {
			$this->cakeError('error', array(array('name' => __('Export failed', true), 'base' => '', 'code' => '',
				'message' => "<ul>\n\t<li>".implode("</li>\n\t<li>", $this->ImportExport->getErrors())."\n</ul>\n")));
			return false;
		}
		
		$this->set('mimetype', $exported['mimetype']);
		if ($withResources) {
			$this->set('zipFile', $exported['zipPath']);
			$this->set('zipName', $exported['zipFileName']);
		} else {
			$this->set('output', $exported['content']);
		}
	}

	public function import() {
		if (! isset($this->params['form']['import_file']))
			return;
		
		if (! $this->ImportExport->import($this->params['form']['method'], $this->params['form']['import_file'])) {
			$this->cakeError('error', array(array('name' => __('Import failed', true), 'base' => '', 'code' => '',
				'message' => "<ul>\n\t<li>".implode("</li>\n\t<li>", $this->ImportExport->getErrors())."\n</ul>\n")));
			return false;
		} else
			return $this->flash(__('Import successful', true), '/');
	}
	
	public function orphans() {
		$this->pageTitle = __('Orphan entries', true);
		
		$models = array(
			'Series',
			'Album',
			'Author',
			'Publisher',
			'Collection',
			'Derivative',
			'DerivativeType',
			'Source',
			'Tag',
			'Image',
			'Binding'
		);
		$associations = array(
			array('Album'),
			array('Series'),
			array('Album', 'Derivative'),
			array('Album'),
			array('Album', 'Publisher'),
			array('Series'),
			array('Derivative'),
			array('Derivative'),
			array('Album'),
			array('Album', 'Derivative'),
			array('Album')
		);
		
		$max = count($models);
		$orphanage = array();
		for ($i = 0; $i < $max; $i++) {
			$orphanage[$models[$i]] = $this->{$models[$i]}->find(
				'orphans', array('associations' => $associations[$i]));
		}
		
		// Additional: collections without publisher
		$orphanage['Collection-noPublisher'] = $this->Collection->find(
			'orphans', array('associations' => array('Publisher')));
		
		$this->set('orphanage', $orphanage);
	}
}

?>
