<?php
/* Copyright 2007, 2008 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
/** Provides helper methods for the Shadowbox library and its jQuery adapter.
 * http://mjijackson.com/shadowbox/
 */
class ShadowboxHelper extends AppHelper {
	public $helpers = array('Html', 'Javascript', 'jQuery');
	
	/** @param $wrap True if the code should be wrapped in a Javascript script tag
	 * with jQuery onload event */
	public function init($wrap = true) {
		$this->jQuery->checkInclude();
		$this->Javascript->link('shadowbox-jquery', false);
		$this->Javascript->link('shadowbox', false);
		$spinner = Router::url('/'.IMAGES_URL.'shadowbox-loading-light.gif');
		$overlay = Router::url('/'.IMAGES_URL.'shadowbox-overlay.png');
		$script = <<<EOS
Shadowbox.init({
        loadingImage: '$spinner',
        overlayBgImage: '$overlay',
        overlayOpacity: 0.25,
        continuous: true,
        handleLgImages: 'resize'
    });

EOS;
		if ($wrap)
			$script = <<<EOS
<script type="text/javascript">
jQuery(function() {
$script
});			
</script>
EOS;
		return $script;
	}
	
	public function thumb($imageUrl, $thumbUrl, $options = null) {
		if (isset($options['group']) && $options['group'] != null)
			$rel = "rel=\"shadowbox[{$options['group']}]\"";
		else
			$rel = "rel=\"shadowbox\"";
		
		if (empty($imageUrl)) {
			$imageUrl = '#';
			$rel = '';
		} else if (!isset($options['isLocal']) || $options['isLocal'])
			$imageUrl = Router::url('/'.IMAGES_URL.$imageUrl);
		
		$options['alt'] = isset($options['alt'])?$options['alt']:__('Thumbnail', true);
		$options['class'] = isset($options['class'])?$options['class']:'thumb'; 
		
		$legend = isset($options['legend'])?$options['legend']:'';
		
		unset($options['group'], $options['isLocal'], $options['legend']);
		
		if (empty($thumbUrl))
			$thumbUrl = 'default-thumbnail.png';
		return "<div class=\"thumb\"><a href=\"$imageUrl\" $rel>".$this->Html->image($thumbUrl, $options)."</a>$legend</div>\n";
	}
	
	/** Returns HTML for a gallery of images
	 *
	 * @param array $images A list of images to show, each elemnt being a hash
	 * containing at least 'url', 'thumb_url' and 'description'. 'legend' can
	 * also be supplied.
	 * @param string $galleryName The name for gallery elements, random by
	 * default
	 * @param integer $perLine The number of items per line, defaults to
	 * the setting in the configuration.
	 * @return string
	 */
	public function gallery($images, $galleryName = null, $perLine = 0) {
		$total = count($images);
		if ($total < 1)
			return;
		
		if ($galleryName === null)
			$galleryName = 'gal'.rand(0, 10000);
		if ($perLine === 0)
			$perLine = Configure::read('Demyo.Gallery_thumbsPerLine');
		
		$gallery='';
		$i = 0;
		foreach($images as $image) {
			$gallery .= $this->thumb($image['url'], $image['thumb_url'],
					array('title' => $image['description'],
						'group' => $galleryName,
						'legend' => isset($image['legend'])?$image['legend']:''
					));
			if ((++$i % $perLine) === 0)
				$gallery.='<div class="clearer"></div>';
		}
		$gallery.='<div class="clearer"></div>';
		
		return $gallery;
	}
}
?>