/* Copyright 2007, 2008 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
/*
jQuery delayed event: Only fire the bound event after a given timeout.

Warning: the callback shouldn't use 'this' to locate the object that got the
event, but rather the second parameter that will be passed. This is due to the
fact that the delayed event is in fact triggered by the window ('this' is then
associated to the window).
The bahavior is thus different than the hover() function and if you want your
function to work in both cases, you'll have to check if the second parameter is
defined before using 'this'.
If anyone finds a cleaner workaround, drop me a not.
*/

(function() {
jQuery.fn.extend({
	delayedEvent: function(eventName, callback, settings) {
		if (! this.length)
			return this;
		
		settings = jQuery.extend({
			timeout: 400 /* Might even be fast for the average user */
		}, settings);
		
		return this.each(function(){
			if (typeof(this.timeout) == "undefined")
				this.timeout = Array();
			this.timeout[eventName] = null;
			
			var thisTimeout = this.timeout;
			
			function delayedHandler(e) {
				if (thisTimeout[eventName])
					thisTimeout[eventName] = clearTimeout(thisTimeout[eventName]);
				
				var subject = this;
				thisTimeout[eventName] = setTimeout(function(){ callback(e,subject); }, settings.timeout);
			}
			
			return jQuery(this).bind(eventName, delayedHandler);
		});
	}
});

})();

