/* Copyright 2007, 2008 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
/* Register an input field as a number field.
 * This plugins assumes that your input field(s) has an id
 *
 * Configurable settings:
 *		min: the minimal value (def 0)
 *		max: the maximal value (def 0xFFFF)
 *		default: the default value (def '')
 *		increment: the step size to increment or decrement
 * 		allowEmpty: true if the field can be empty (def true)
 * 		allowFloat: true if float number are allowed (def false)
 * 		upText: the text to show as 'up arrow' (def '&uarr;')
 * 		downText: the text to show as 'down arrow' (def '&darr;')
 *
 * Styleable classes: div.numberHelper, .numberHelper span.nhUp, .numberHelper span.nhDown
 * 
 * Depends on jquery.dimensions
 * 
 * Todo: check that the given elements are input fields
 */
(function() {
jQuery.fn.extend({
	numberField: function(settings) {
		// setup configuration
		settings = jQuery.extend({
			min: 0,
			default: "",
			max: 0xFFFF,
			increment: 1,
			allowEmpty: true,
			allowFloat: false,
			upText: '&uarr;',
			downText: '&darr;'
		}, settings);
		
		if (settings.allowFloat)
			settings.regex = /^-?[0-9]+(\.[0-9]*)?$/;
		else
			settings.regex = /^-?[0-9]+$/;
	
		return this.each(setUpField);
		
		function setUpField() {
			var jqSelf = jQuery(this);
			
			// Check values
			checkField(jqSelf);
			jqSelf.bind('change', function() { checkField(jQuery(this)) });
			
			// Add incrementors
			var top = jqSelf.offset().top;
			var left = jqSelf.offset().left+jqSelf.outerWidth();
			var height = jqSelf.innerHeight();
			var id = jqSelf.attr('id');
			jqSelf.after('<div class="numberHelper" id="nh_'+id+'" style="position: absolute; top: '+top+'px; left: '+left+'px; height: '+height+'px">'
				+'<span class="nhDown">'+settings.downText+'</span>'
				+'<span class="nhUp">'+settings.upText+'</span>'
			);
			
			// Register incrementors
			jQuery('div#nh_'+id+' span.nhDown').click(decrement);
			jQuery('div#nh_'+id+' span.nhUp').click(increment);
		}
		
		function checkField(jqField) {
			var val = jqField.val();
			matches = val.match(settings.regex);
			if (val == "" && settings.allowEmpty)
				;
			else if (settings.regex.test(val)) {
				if (val < settings.min)
					jqField.val(settings.min);
				else if (val > settings.max)
					jqField.val(settings.max);
			}
			else
				jqField.val(settings.default);
		}
		
		function decrement(ev) {
			edit(jQuery(ev.target).parent().attr('id').substr(3), -1);
		}
		
		function increment(ev) {
			edit(jQuery(ev.target).parent().attr('id').substr(3), 1);
		}
		
		function edit(fieldID, factor) {
			var jqField = jQuery('#'+fieldID);
			var number = NaN;
			if (settings.allowFloat)
				number = parseFloat(jqField.val());
			else
				number = parseInt(jqField.val());
			
			if (isNaN(number))
				number = settings.default;
			else
				number = number + factor * settings.increment;
			
			jqField.val(number);
			jqField.trigger('change');
		}
	}
});

})();