/* Copyright 2007, 2008 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
/*
jQuery.slider

Inspired by jQuery.imageMenu <http://ldbglobe.fried-rice.net/public/2007/06/jQuery.imageMenu/>,
which itself took its inspiration from phatfusion image Menu
<http://www.phatfusion.net/imagemenu/index.htm>

There's hardly anything left from jQuery.imageMenu, and I doubt that there's a
patent on phatfusion's so I can probably relicence it.

usage: $('ul#foo').slider(settings)
settings is a hash map with the following keys:
	easing	The easing to use for animations
	speed	The animation speed
	extend	The size each element should be extended (i.e. on hover, an element
			has a width of his optimal width + extend)
			Set to -1 to try to compute it automatically, in which case
			computing will be based on the width the element would have if it
			was not constrained (works best for images or elements having a
			min-width)
*/

(function() {
jQuery.fn.extend({
	slider: function(settings) {
		if (! this.length)
			return this;
		
		var listTotalWidth, optimalWidth, hoverMaxWidth, hoverMinWidth;
		var _ = this;
		var originalWidthRatio = jQuery(this).width() / jQuery(jQuery(this).parent().get(0)).width();
		var children = jQuery('li', this);
		
		// The following allows us to clip the overflowing elements rather
		// than overflow them. Saves some computation since the actual width
		// of an element takes the borders into account
		this.css('position','relative');
		this.append('<div style="position:absolute;height:'
			+this.height()+'px;" class="containerJKjFJN"></div>'); // name should avoid conflicts with user-specified CSS
		children.appendTo(jQuery('.containerJKjFJN',this));
		
		// setup configuration
		settings = jQuery.extend({
			easing: 'swing',
			speed: 'fast',
			extend: -1
		}, settings);
		
		if (! this.is('ul')) // only ul's are supported
			return this;
		
		this.css('list-style','none');
		this.css('overflow','hidden');
		children.css('float','left');
		
		function mouseOver(a, e) {
			// Handle both normal hover and delayed event
			if (typeof(e) == 'undefined')
				e = this;
			children.stop();
			children.animate({'width':hoverMinWidth}, settings.speed, settings.easing);
			jQuery(e).stop();
			jQuery(e).animate({'width':hoverMaxWidth}, settings.speed, settings.easing);
		}
		
		function mouseOut() {
			children.stop();
			children.animate({'width':optimalWidth}, settings.speed, settings.easing);
		}
		
		function noop() {}
		
		function resetWidths() {
			// Reset original width ratio since we maybe changed it before resize occured
			jQuery(_).width(originalWidthRatio*jQuery(jQuery(_).parent().get(0)).width());
			
			listTotalWidth = jQuery(_).width();
			optimalWidth = Math.floor(listTotalWidth / children.length);
			
			// Find out what he largest width is
			var maxWidth = 0;
			children.each(
				function(i) {
					if (jQuery('img', this).width() > maxWidth)
						maxWidth = jQuery('img', this).width();
				}
			);
			/*alert('parent: '+jQuery(jQuery(_).parent().get(0)).width()+"\n"
				+ 'ratio: '+originalWidthRatio+"\n"
				+'-> width: '+listTotalWidth+"\n\n"
				+'1 children: '+maxWidth+"\n"
				+'all children: '+maxWidth*children.length);*/
			
			// If the list is too large, crop it to avoid blank space
			if (maxWidth*children.length < listTotalWidth) {
				jQuery(_).width(maxWidth*children.length+10);
				hoverMinWidth = optimalWidth = maxWidth;
				children.width(optimalWidth); // TODO BUG: this makes the last children disappear
				return;
			}
			
			// The list is too small. We can auto-compute the extend width...
			else if (settings.extend == -1)
				hoverMaxWidth = maxWidth;
			// ... or we have a given extend width
			else
				hoverMaxWidth = optimalWidth + settings.extend;
			
			hoverMinWidth = Math.floor((listTotalWidth - hoverMaxWidth)/(children.length-1));
			
			/*DEBUG
			alert("width: " + listTotalWidth
				+ "\noptimal: " + optimalWidth
				+ "\napproximation: " + optimalWidth*children.length
				+ "\nhover max: " + hoverMaxWidth
				+ "\nhover min: " + hoverMinWidth
				+ "\nhover total: " + (hoverMinWidth*(children.length-1)+hoverMaxWidth)
				);
			*/
			
			// Set the children to the right width. Apparently, clipping is better
			// handled if we first animate to the minimum, then stop and set to
			// optimal
			children.animate({'width':hoverMinWidth}, settings.speed, 'swing');
			children.stop();
			children.width(optimalWidth);
			
			jQuery('.containerJKjFJN').width(listTotalWidth*2);
		}
		
		resetWidths();
		
		// Don't want the dependency on delayedEvent ? Switch the comments
		children.delayedEvent('mouseover', mouseOver, {timeout: 100});
		this.delayedEvent('mouseout', mouseOut, {timeout: 100});
		//children.hover(mouseOver, noop);
		//this.hover(noop, mouseOut);
		
		jQuery(window).resize(resetWidths);
		return this;
	}
});

})();

